//
// CfgParser.hh
// Copyright (C) 2003-2005 Claes Nasten <pekdon{@}pekdon{.}net>
//                         Alaa Abd El Fatah <alaa{@}linux-egypt{.}org>
//
// This program is licensed under the GNU GPL.
// See the LICENSE file for more information.
//

#include "../config.h"

#ifndef _CFG_PARSER_HH_
#define _CFG_PARSER_HH_

#include "Types.hh"

#include <string>
#include <algorithm>
#include <list>
#include <map>
#include <cstring>
#include <fstream>

class CfgParser
{
public:
	struct Item {
		std::string name;
		std::string value;
		// Well, it seems as if gcc-2.95.X uses != in find and
		// gcc-3.x uses == . So I have to have both
		inline bool operator == (const std::string &s) {
			return (strcasecmp(name.c_str(), s.c_str()) ? false : true);
		}
		inline bool operator != (const std::string &s) {
			return (strcasecmp(name.c_str(), s.c_str()));
		}
	};

	class Section {
	public:
		Section(std::list<std::string> &sources,
						std::map<std::string, std::string> &variables,
						const std::string &name = "") : _name(name),
																						_sources_list(sources),
																						_variables(variables) { }
		~Section(void) { }
		void load(std::istream& file);
		bool load(const std::string& filename);
		inline const std::string& getName(void) { return _name; }

		inline Section* getSection(const std::string& name) {
			std::list<Section>::iterator it =
				find(_sect_list.begin(), _sect_list.end(), name);
			if (it != _sect_list.end())
				return &*it;
			return NULL;
		}

		inline Section* getNextSection(void) {
			if (_sect_it != _sect_list.end())
				return &*_sect_it++;
			return NULL;
		}

		bool getValue(const std::string &name, int &value);
		bool getValue(const std::string &name, uint &value);
		bool getValue(const std::string &name, bool &value);
		bool getValue(const std::string &name, std::string &value);
		bool getNextValue(std::string &name, std::string &value);

		// Well, it seems as if gcc-2.95.X uses != in find and
		// gcc-3.x uses == . So I have to have both
		inline bool operator == (const std::string &s) {
			return !(strcasecmp(_name.c_str(), s.c_str()));
		}
		inline bool operator != (const std::string &s) {
			return (strcasecmp(_name.c_str(), s.c_str()));
		}

	private:
		std::string _name;
		std::list<Item> _values;
		std::list<Item>::iterator _it;
		std::list<Section> _sect_list;
		std::list<Section>::iterator _sect_it;

		std::list<std::string> &_sources_list; // list of sources
		std::map<std::string, std::string> &_variables; // reference to variables
	};

	CfgParser(void) : _cfg(_sources_list, _variables) { }
	~CfgParser(void) { }

	bool load(const std::string &file);
	bool loadCommand(const std::string &command);

	inline Section* getSection(const std::string& name) {
	  return _cfg.getSection(name);
	}
	inline Section* getNextSection(void) {
	  return _cfg.getNextSection();
	}

private:
  Section _cfg;
	std::list<std::string> _sources_list; // list of sources
	std::map<std::string, std::string> _variables; // map for $variables
};

#endif // _CFG_PARSER_HH_
