//
// RegexString.hh for pekwm
// Copyright (C)  2003-2005 Claes Nasten <pekdon{@}pekdon{.}net>
//
// This program is licensed under the GNU GPL.
// See the LICENSE file for more information.
//

#include "../config.h"

#ifndef _REGEX_STRING_HH_
#define _REGEX_STRING_HH_

#include "Types.hh"
 
#include <string>
#include <list>

extern "C" {
#include <sys/types.h>
#ifdef HAVE_PCRE
#include <pcreposix.h>
#else // !HAVE_PCRE
#include <regex.h>
#endif // HAVE_PCRE
}

//! @brief POSIX regular extensions wrapper with support for PCRE.
class RegexString
{
public:
	//! @brief RegListItem, class used to store RegexString::replace data.
	class RegListItem
	{
	public:
		//! @brief RegListItem constructor, initialises _string and _ref.
		RegListItem(const std::string &nstring, int ref) :
			_string(nstring), _ref(ref) { }
		~RegListItem(void) { }

		//! @brief Returns string item represents if any.
		const std::string& getString(void) { return _string; }
		//! @brief Returns reference number item represents if any.
		int getRef(void) { return _ref; }

	private:
		std::string _string;
		int _ref;
	};

	RegexString(void);
	RegexString(const std::string &exp_ref);
	RegexString(const std::string &exp, const std::string &ref);
	~RegexString(void);

	//! @brief Returns true if RegexString::parse went ok.
	inline bool isOk(void) const { return _ok; }
	//! @brief Returns true if regular expression compiled ok.
	inline bool isRegOk(void) const { return _reg_ok; }
	//! @brief Returns true if reference replacing parsed ok.
	inline bool isRefOk(void) const { return _ref_ok; }

	bool parse(const std::string &exp_ref);
	bool replace(std::string &rhs);

	//! @brief Calls parseReg with right hand string as parameter.
	inline RegexString &operator = (const std::string &rhs) {
		parseReg(rhs);
		return *this;
	}
	//! @brief Returns true if regular expression matches right hand string.
	inline bool operator == (const std::string &rhs) {
		if (_reg_ok) {
			return (regexec(&_regex, (const char*) rhs.c_str(), 0, 0, 0) == 0);
		}
		return false;
	}

private:
	void parseReg(const std::string &exp);
	void parseRef(const std::string &ref);

	void freeReg(void);

private:
	regex_t _regex;

	uint _ref_max;
	std::list<RegListItem> _ref_list;

	bool _ok, _reg_ok, _ref_ok;

	static const char* NUMBERS;
};

#endif // _REGEX_STRING_HH_
